#  aRT : API R - TerraLib                                                
#  Copyright (C) 2003-2005  LEG                                          
#                                                                        
#  This program is free software; you can redistribute it and/or modify  
#  it under the terms of the GNU General Public License as published by  
#  the Free Software Foundation; either version 2 of the License, or     
#  (at your option) any later version.                                   
#                                                                        
#  This program is distributed in the hope that it will be useful,       
#  but WITHOUT ANY WARRANTY; without even the implied warranty of        
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
#  GNU General Public License for more details.                          
#                                                                        
#  You should have received a copy of the GNU Lesser General Public      
#  License along with this library.

# aRT.R: calls C++ functions of main.cpp.
# All this functions only check for user argument errors.
# Other errors are checked in the next layers.

setClass("aRTquerier", contains = "aRTcomponent")

setMethod("initialize",
          "aRTquerier",
		  function(.Object,
                   parent,
                   loadgeom = FALSE,
                   loadatt = TRUE,
				   table = "",
				   att = "",
				   chronon = c("nochronon", "second", "minute",
							   "hour", "day", "month", "year",
    						   "dayofweek", "dayofmonth", "dayofyear",
							   "monthofyear", "season", "weekofyear",
							   "hourofday", "minuteofhour", "secondofminute"),
				   stat = c("nostat", "sum", "max", "min",
							"count", "validcount", "stddev",
							"kernek", "mean", "variance",
							"skewness",	"kurtosis",	"median",
							"varcoeff",	"mode"))
{

  stat    = match.arg(stat)
  chronon = match.arg(chronon)

  if (is.object(parent) && class(parent) == "aRTlayer")
  {
    if(stat    != "nostat")    stop("Can't use stat when building a query from layer\n")
    if(chronon != "nochronon") stop("Can't use chronon when building a query from layer\n")
    out = .Call("cppProxy",
                parent@pointer,
                list(loadgeom=loadgeom,
                     loadatt=loadatt),
                PACKAGE="aRT")
  }
  else if (is.object(parent) && class(parent) == "aRTtheme")
  {
	sum = summary(parent)

	if(table != "" && !any(names(sum$tables) == table)) stop("No table choosed\n")
	if(att   != "" && !any(sum$tables[[table]] == att)) stop("Attribute inexistent\n")
  
	out = .Call("cppOpen",
	            parent@pointer,
				list(loadgeom=loadgeom,
				     loadatt=loadatt,
					 stat=stat,
					 chronon=chronon,
					 table=table,
					 att=att),
				PACKAGE="aRT")
  }
  else stop("Argument \"parent\" must an object of class aRTlayer or aRTtheme\n")

  if (is.null(out))
    stop("Could not create the aRTquerier object\n")

  .Object@pointer <- out
  reg.finalizer(.Object@pointer, .aRTremove)
  .Object
})

setGeneric("nextSlice", function(object) standardGeneric("nextSlice"))

setMethod("nextSlice", "aRTquerier", function(object) {
 .Call("cppNextSlice",
       object@pointer,
       PACKAGE="aRT")
})

setGeneric("openQuerier", function(object, ...) standardGeneric("openQuerier"))

setMethod("openQuerier", "aRTlayer", function(object, loadgeom = FALSE,  loadatt = TRUE) {
	new("aRTquerier", object, loadatt=loadatt, loadgeom=loadgeom)
})

setMethod("openQuerier", "aRTtheme", function(object, ...) {
	new("aRTquerier", object, ...)
})

createQuerier = function(...) openQuerier(...)

