\name{aRTlayer}

\docType{class}

\alias{aRTlayer}
\alias{aRTlayer-class}
\alias{initialize,aRTlayer-method}
\alias{plot,aRTlayer-method}

\title{Class "aRTlayer"}
\description{Class that represents a layer of information in the database. It can insert
geometries into the database, process them and create \code{aRTtable}, \code{aRTtheme} 
and \code{aRTquerier} objects.
TerraLib layers can stores various geometries in a layer, but in aRT, for simplify,
each layer stores only one type of geometry.}

\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("aRTlayer", aRTdb, layername, proj)}.
Examples:\cr
   obj = new("aRTlayer", aRTdb, layername, create=T)\cr
   obj = new("aRTlayer",db,l="llatlong",proj="+proj=latlong +datum=NAD83",c=T)\cr

To avoid creating objects this way, there are two functions of class \code{aRTdb}:
\code{createLayer} and \code{openLayer}.
}

\section{Arguments}{
   \describe{
   \item{aRTdb:}{Object of aRTdb class.}
   \item{layername:}{Name of the layer.}
   \item{proj:}{Projection in proj.4 format. Some projections are not supported by TerraLib.}
   \item{create:}{Can the layer be created if it does not exist?}
   }
}
				 

\section{Slots}{
  \describe{
    \item{\code{pointer}:}{An external pointer. DO NOT change it!}
  }
}

\section{Extends}{
Class \code{"aRTcomponent"}, directly.
}

\section{Methods}{
  \describe{
    \item{addLines:}{Inserts lines in the layer.}
    \item{addPoints:}{Inserts points in the layer.}
    \item{addPolygons:}{Inserts polygons (rings) in the layer.}
    \item{initialize:}{Initializer of aRTlayer class.}
    \item{showTables:}{Shows all information tables in the layer}
	\item{getAttributes:}{Returns the stored attributes.}
	\item{getGeometry:}{Returns the geometries stored in the layer.}
	\item{getMetric:}{Returns a metric operations (buffer, length, area, convex hull).}
	\item{getSetMetric:}{Calculates a set operation (union, difference, intersection, xor).}
	\item{getNN:}{(not implemented yet)}
	\item{getRelation:}{Looks for spatial relations as contains, crosses, touches.}
 	\item{openTheme:}{Opens an existent theme.}
	\item{createTheme:}{Creates a new theme and returns it.}
	\item{plot:}{plot}
 }
}

\references{\url{http://www.est.ufpr.br/aRT}}
\author{Marcos Aurelio Carrero and Pedro Ribeiro de Andrade Neto}
\keyword{classes}
\seealso{
 See \code{aRTlayer-METHOD} for a method of this class or \code{CLASSNAME-class} for links to other classes.
 \pkg{spproj} for conversion of projections.
}

% vim: syntax=tex
