# Exemplo de relacoes espaciais utilizando os
# trs tipos de geometria.
# Este exemplo utiliza os dados de Recife
#
# (C) 2005 LEG

require(aRT)

con = openConn()#new("aRT")

if(any(showDbs(con)=="recife")) deleteDb(con, "recife", force=T)

db = createDb(con, "recife")

lbairros    = createLayer(db, "bairros")
larmadilhas = createLayer(db, "armadilhas")
ldrenagem   = createLayer(db, "drenagem")

data(recife.districts)
data(recife.points)
data(recife.sewers)

addPolygons(lbairros,    recife.districts)
addPoints  (larmadilhas, recife.points)
addLines   (ldrenagem,   recife.sewers)

tbairros    = createTable(lbairros,   "t_bairros")
tarmadilhas = createTable(larmadilhas,"t_armadilhas")
tdrenagem   = createTable(ldrenagem,  "t_drenagem")

# isto demora um pouco... pegar as geometrias dos layers
# utilizados para nao precisar fazer mais
getGeometry(lbairros)   ->geobairros
getGeometry(larmadilhas)->geoarmadilhas
getGeometry(ldrenagem)  ->geodrenagem

par(mfcol=c(2,2), mar=c(3,3,.5,.5))

# plotando bairros e armadilhas
plot(geobairros, col="red")
plot(geoarmadilhas, add=T)

# plotando rede de drenagem da cidade
plot(geobairros, col="gray", lty=0)
plot(geodrenagem, add=T, col="blue")

## Relacoes espaciais
# 1. todos os bairros que Contem armadilhas
getRelation(lbairros, "contains", larmadilhas)->rel1

# 2. todos os vizinhos (touch) dos bairros que possuem armadilhas
getRelation(lbairros, "touch", id=rel1)->rel2

plot(geobairros, col="gray", lty=0)
plot(geobairros, col="green", add=T, id=rel2)
plot(geobairros, col="yellow", add=T, id=rel1)
# nao desenhei as armadilhas porque senao nao d pra ver os bairros (fica preto)
#plot(geoarmadilhas, add=T)

# 3. drenagens que cruzam os bairros que tem armadilhas
getRelation(ldrenagem, "crosses", layer=lbairros, id=rel1)->rel3

# 4. drenagens que estao contidas (within) nos bairros que tem armadilhas
getRelation(ldrenagem, "within", layer=lbairros, id=rel1)->rel4

plot(geobairros, col="gray", lty=0, id=rel1)
plot(geobairros, col="gray", lty=0, add=T)
plot(geobairros, col="yellow", lty=0, id=rel1, add=T)
plot(geodrenagem, add=T)
plot(geodrenagem, add=T, col="green", id=rel3)
plot(geodrenagem, add=T, col="red", id=rel4)

# gerando temas com as mesmas imagens vistas no R
th1 = createTheme(lbairros,    "bairros1",    view="bairros")
th2 = createTheme(larmadilhas, "armadilhas1", view="bairros")

setVisual(th1, visualPolygons(color="red"))
setVisual(th2, visualPoints(pch=22))

setVisible(th1)
setVisible(th2)

th3 = createTheme(lbairros,  "bairros2",  view="drenagem")
th4 = createTheme(ldrenagem, "drenagem2", view="drenagem")

setVisual(th3, visualPolygons(color="gray", ccolor="gray"))
setVisual(th4, visualLines(color="blue"))

setVisible(th3)
setVisible(th4)

th5 = createTheme(lbairros, "vizinhos3", view="vizinhos", id=unique(rel2))
th6 = createTheme(lbairros, "contem3",   view="vizinhos", id=unique(rel1))
th7 = createTheme(lbairros, "todos3",    view="vizinhos")

setVisual(th5, visualPolygons(color="green"))
setVisual(th6, visualPolygons(color="yellow"))
setVisual(th7, visualPolygons(color="gray", ccolor="gray"))

setVisible(th5)
setVisible(th6)
setVisible(th7)

th8  = createTheme(ldrenagem, "cruzam4",   view="final", id=unique(rel3))
th9  = createTheme(ldrenagem, "contem4",   view="final", id=unique(rel4))
th10 = createTheme(lbairros,  "vizinhos4", view="final", id=unique(rel1))
th11 = createTheme(ldrenagem, "drenagem4", view="final")
th12 = createTheme(lbairros,  "bairros4",  view="final")

setVisual(th8, visualLines(color="blue"))
setVisual(th9, visualLines(color="red"))
setVisual(th10, visualPolygons(color="yellow", ccolor="yellow"))
setVisual(th11, visualLines(color="black"))
setVisual(th12, visualPolygons(color="gray", ccolor="gray"))

setVisible(th8)
setVisible(th9)
setVisible(th10)
setVisible(th11)
setVisible(th12)

